
import java.sql.*;
/**
 * Beispiel einer Datenbankverbindung zur Datenbank highscore.db - Tabelle bestenliste.
 * 
 * @author Klaus Reinold 
 * @version 1.0
 */
class Datenbankanbindung
{
    /** Objekt zur Verwaltung der Verbindung */
    Connection verbindung;

    /**
     * Konstruktor
     */
    Datenbankanbindung()
    {

        try
        {
            //Registrierung des Datenbanktreibers - kann nach der ersten Benutzung auskommentiert werden. 
            Class.forName("org.sqlite.JDBC").newInstance();
            //Pfad zur Datenbank (Datei highscore.db im Projektordner) 
            //über die Schnittstelle JDBC (Java Database Connectivity)
            //unter Nutzung eines Treibers für SQLite Datenbanken
            String url = "jdbc:sqlite:highscore.db";
            // Aufbau der Verbindung zur Datenbank
            verbindung = DriverManager.getConnection(url);

            System.out.println("Verbindung zur Datenbank aufgebaut.");
        }
        catch (Exception ex)
        {
            System.out.println("Exception: " +ex.getMessage());
        }

    }

    /**
     * Gibt die Bestenliste aus.
     */
    void ErgebnistabelleAusgeben()
    {
        try
        {
            //Erzeugen eines Anfrage-Objekts
            Statement anfrage = verbindung.createStatement();
            // Abschicken der Anfrage; als Antwort kommt die Ergebnistabelle.
            ResultSet ergebnistabelle = anfrage.executeQuery("SELECT name, punkte FROM bestenliste ORDER BY punkte DESC");

            //Rechteck zur Darstellung der Highscore-Liste
            Rechteck r;
            r = new Rechteck();
            r.FarbeSetzen("gelb");
            r.PositionSetzen(10, 230);
            r.GrößeSetzen(200, 220);

            //Textfeld zur Darstellung der Bestplatzierten
            Text top10;

            int i = 1;
            // Iteration über die Ergebnistabelle (10 Bestplatzierte)
            while(ergebnistabelle.next() && i<=10)
            {
                top10 = new Text();
                System.out.println(ergebnistabelle.getString("name"));
                top10.TextSetzen("Platz "+ i+ ": "+ergebnistabelle.getString("name")+ " Punkte: "+ergebnistabelle.getString("punkte"));
                top10.PositionSetzen(20, 240+ 20*i);
                i = i + 1;
            }

            //Beenden
            ergebnistabelle.close();
            anfrage.close();
        }
        catch(Exception e)
        {
            System.out.println(e.getMessage()); 
        }

    }

    /**
     * Eintragen eines Datensatzes in die Ergebnistabelle
     * @param nameNeu Name des Spielers
     * @param punkteNeu Punktestand
     */
    void DatensatzEintragen(String nameNeu, int punkteNeu)
    {
        try
        {
            Statement anfrage = verbindung.createStatement();
            anfrage.executeUpdate("INSERT INTO bestenliste(name, punkte) VALUES ('" + nameNeu + "', " + punkteNeu + ")");
            anfrage.close(); 
        }
        catch(Exception e)
        {
            System.out.println(e.getMessage());
        }
    }
    
    /**
     * Schließt die Datenbankverbindung
     */
    void VerbindungSchließen()
    {
        try
        {
            verbindung.close();
        }
        catch (Exception e)
        {
            System.out.println(e.getMessage());
        }
    }
}
